package client_server;

/**
 * The Request class uses the StringTokenizer created in the SimpleWebServer class to
 * create a Properties list of the parameters encoded in the URL string.  
*/
import java.io.*;
import java.net.*;
import java.util.*;

public class Request
{
	private Properties list;
	private Item [] itemList;
	private int size = 0;
	private String method;
	
	/* Use the tokenizer to get the name and value parameters and 
	 * store them in the Properties list.  
	*/
	public Request (StringTokenizer tokenizer, String method)
	{
		String name, value;
		this.method = method;
		itemList = new Item [10];
		list = new Properties ();
		while (tokenizer.hasMoreTokens () && (size < 10))
		{
			name = tokenizer.nextToken ();
			value = tokenizer.nextToken ();
			System.out.println (name + " " + value);
			if (!name.equals ("HTTP"))
			{
				value = replaceHexCode (value);
				list.setProperty (name, value);
				Item item = new Item (name, value);
				itemList [size] = item;
				size ++;
			}
		} 
	} // constructor 
	
	/* Some characters are sent in hex by the URL string.  They are preceded by a '%' sign.
	 * The following method replaces them with the corresponding characters.  It also replaces
	 * the '+' sign in the string with a space.
	*/
	private String replaceHexCode (String value)
	{
		value = value.replace ('+', ' ');	
		int index = value.indexOf ('%');
		while (index >= 0)
		{	try 
			{
				// Get the hex code and covert it to decimal.
				String hex = value.substring (index+1, index+3);
				int decimal = Integer.parseInt (hex, 16);
			
				/* Get the character with the decimal code, change the characters '<' to &lt,
				and '>' to &gt.  Include the resulting string in the value parameter. */
				char ch = (char) decimal;
				String code;
				if (ch == '<') code = "&lt";
				else if (ch == '>') code = "&gt";
				else code = String.valueOf (ch);
				value = value.substring (0, index) + code + value.substring (index+3);
			} catch (NumberFormatException e) {}
			index = value.indexOf ('%', index+1);
		}
		return value;
	} // replaceHexCode
	
	// Given a name, return the corresponding value.
	public String getParameter (String name)
	{
		return list.getProperty (name);
	} // getParameter
	
	public String getMethod () {return method;}
	
	// Return an array containing just the parameter values, not the names.
	public String [] getParameterValues (String paramName)
	{
		String [] values = new String [10];
		int count = 0;
		while (count < size)
		{
			if (itemList [count].getName ().equals (paramName))
				values [count] = itemList [count].getValue ();
			count ++;
		}
		return values;
	} // getParameterValues

} // class Request

class Item
{
	private String name, value;
	
	Item (String n, String v)
	{
		name = n;
		value = v;
	}
	
	protected String getName () {return name;}
	protected String getValue () {return value;}
}
			